/*  This is to emulate another remote process also using socket io to connect.
TEST: to see if it performs the necessary self check to identify itself, as well as 
receiving all the notification and response messages */

// Import the necessary modules
import { io, Socket } from "socket.io-client";
import { handleClientSocketConnection } from "../utils/socket.utils";
import { ConnectedServerSocket } from "../transport/websocket";
import { v4 as uuidv4 } from 'uuid'
import { interval, Subject } from "rxjs";
import { TransportEvent, TransportMessage } from "../interface/connector.interface";
import { FisMessage } from "../interface/transport.interface";

class SocketClient {
    private socket: Socket;
    private connectedServerSocket: ConnectedServerSocket[] = []
    private event: Subject<TransportEvent> = new Subject()

    constructor(url: string) {
        // Connect to the serve
        this.socket = io(url);
        // use the existing socket handler
        handleClientSocketConnection(this.socket, this.connectedServerSocket).subscribe(this.event)
        this.startListening(this.event)
        this.sendMessage()
    }

    private startListening(event: Subject<TransportEvent>): void {
        event.subscribe((event: TransportEvent) => console.log(event))
    }

    private sendMessage(): void {
        interval(3000).subscribe(interval => {
            let message: FisMessage = {
                header: {
                    messageID: uuidv4(),
                    messageName: 'RequestMessage'
                }, 
                data: 'Data'
            }
            this.socket.emit('message', message)
        })
    }
}

// Usage example:
const client = new SocketClient("http://localhost:3000");

// Send a message
