import { filter, map, Observable, Observer, PartialObserver, Subject, Subscriber, Subscription, takeWhile, Unsubscribable } from 'rxjs';
import { AdapterProfile, AdapterSet, AdaptorTransmissionRole, Transport, TransportEvent, TransportMessage } from '../interface/connector.interface';
import { MessageTransmissionBase } from './msg.transmission.base';
import { Bus, EventMessage, FisMessage, MessageReceiver as MessageReceiverInterface, ReceiverProfile, TransmissionMessage, TransmitterProfile } from '../interface/transport.interface'
import { ConnectionAdapter } from '../connector/connector.base';
import { v4 as uuidv4 } from 'uuid'
import { ReceiverConnectionAdapter } from '../connector/connector.receiver';
import { ConnectionManager } from '../connector/connector.manager';

export class MessageTransmissionReceiver extends MessageTransmissionBase implements MessageReceiverInterface {
    receiverProfile!: ReceiverProfile;

    constructor(profile: ReceiverProfile, adapterSets: AdapterSet[], event: Observable<TransportEvent>) {
        super(event);

        this.setReceiver(profile)
        this.setUpAdapter(adapterSets)
    }

    setReceiver(receiverProfile: ReceiverProfile): void {
        this.receiverProfile = receiverProfile
    }

    getMessageBus(bus: Bus): Observable<TransmissionMessage> {
        return new Observable((observable: Observer<TransmissionMessage>) => {
            // logic here
            if (bus == Bus.GeneralBus) {
                // Need to merge all the adapters into one when the time comes 
                // SAMPLE: This adapterArray.forEach(adapter => { ... })
                const subscription: Subscription = (this.adapterService as ReceiverConnectionAdapter).getMessageBus(Bus.GeneralBus).pipe(
                    filter((event: TransportEvent) => event.event == 'New Message'),
                    map((event: TransportEvent) => event.data as TransportMessage)
                ).subscribe((message: TransportMessage) => {
                    observable.next({
                        adapterId: message.target as string, // assign adapter id so that it knows who to reply back
                        payload: message.payload
                    });
                });

                // Clean up on unsubscription
                return () => {
                    subscription.unsubscribe();
                };
            }
        })
    }

    setUpAdapter(adapterSets: AdapterSet[]): void {
        if (adapterSets.length > 0) {
            adapterSets.forEach((adapter: AdapterSet) => {
                this.adaptorsArray.push(adapter.receiverAdapter)
            })
        }
        // for now just hardcode to use 1 adapter type until connection manager is further enhacne to configure adapters on the fly
        this.adapterService = this.adaptorsArray.find(obj => obj.getInfo().transportType === Transport.Websocket)
    }
}
