import { MessageTransmissionBase } from "./msg.transmission.base";
import { FisMessage, MessageTransmitter as MessageTransmitterInterface, TransmitterProfile } from '../interface/transport.interface'
import { AdapterSet, Transport, TransportEvent } from "../interface/connector.interface";
import { v4 as uuidv4 } from 'uuid'
import { TransmitterConnectionAdapter } from "../connector/connector.transmitter";
import { Observable } from "rxjs";

/* Take in all the messages that needs to be transported, and divide them accordingly. So the connector instances will do just that
connectors or adapters will have their own identifier*/
export class MessageTransmissionTransmitter extends MessageTransmissionBase implements MessageTransmitterInterface {
    transmitterProfile!: TransmitterProfile;

    constructor(profile: TransmitterProfile, adapterSets: AdapterSet[], event: Observable<TransportEvent>) {
        super(event)
        this.setTransmitter(profile)
        this.setUpAdapter(adapterSets)

    }

    setTransmitter(transmitterProfile: TransmitterProfile): void {
        this.transmitterProfile = transmitterProfile
    }

    emit(message: FisMessage): void {
        // for now only use one adapter
        (this.adapterService as TransmitterConnectionAdapter).emit(message)
    }

    setUpAdapter(adapterSets: AdapterSet[]) {
        if (adapterSets.length > 0) {
            adapterSets.forEach((adapter: AdapterSet) => {
                this.adaptorsArray.push(adapter.transmitterAdapter)
            })
        }
        // for now just hardcode to use 1 adapter type until connection manager is further enhacne to configure adapters on the fly
        this.adapterService = this.adaptorsArray.find(obj => obj.getInfo().transportType === Transport.Websocket)
    }
}